"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var isFinite_1 = require("lodash/isFinite");
var Query = /** @class */ (function () {
    function Query(query) {
        this._etiquetas = [];
        this._descripcion = [];
        this._modificadores = [];
        this._query = '';
        this._queryGenerador = '';
        this.addQuery(query);
    }
    Object.defineProperty(Query.prototype, "etiquetas", {
        get: function () {
            return this._etiquetas;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Query.prototype, "descripcion", {
        get: function () {
            return this._descripcion;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Query.prototype, "query", {
        get: function () {
            return this._query;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Query.prototype, "queryGenerador", {
        get: function () {
            return this._queryGenerador;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Método para obtener las etiquetas de un search query
     * @param query
     */
    Query.prototype.parseQueryEtiquetas = function (query) {
        var regex = /[A-Za-z][\.a-zA-Z0-9_]+(?=:)/g;
        var etiquetas = [];
        var match;
        do {
            match = regex.exec(query);
            if (match) {
                etiquetas.push(match[0]);
            }
        } while (match);
        return etiquetas;
    };
    /**
     * Metodo para obtener la descripcion de las etiquetas
     *
     * @param {string} query
     * @returns {string[]}
     * @memberof Query
     */
    Query.prototype.parseQueryDescripcion = function (query) {
        var regex = /[a-zA-Z0-9-_ &/]+(?![\.a-zA-Z_0-9]*:)/g;
        var descripcion = [];
        var match;
        do {
            match = regex.exec(query);
            if (match) {
                descripcion.push(match[0].trim());
            }
        } while (match);
        return descripcion;
    };
    Query.prototype.parseQueryModificador = function (query) {
        var regex = /(:>=)|(:<=)|(:>)|(:<)|(:=)|(:#)|(:\+)|(:)/g;
        var modificador = [];
        var match;
        do {
            match = regex.exec(query);
            if (match) {
                modificador.push(match[0].trim().substr(1));
            }
        } while (match);
        return modificador;
    };
    /**
     * Construye el query de busqueda
     *
     * @param {string[]} etiquetas
     * @param {string[]} descripcion
     * @param {string[]} modificadores
     * @returns {string}
     * @memberof Query
     */
    Query.prototype.construccionQuery = function (etiquetas, descripcion, modificadores) {
        var etiquetasConCategoria = etiquetas.map(function (etiqueta, indice) {
            // Comprobar si es fecha segun formato inicio-fin: YYYY/MM/DD-YYYY/MM/DD ó YYYY/MM/DD
            var regexFecha = /[0-9]{4}\/[0-9]{2}\/[0-9]{2}-[0-9]{4}\/[0-9]{2}\/[0-9]{2}|[0-9]{4}\/[0-9]{2}\/[0-9]{2}/g;
            var regexIn = /&/g;
            var isFormatoFecha = regexFecha.exec(descripcion[indice]);
            var isMultipleInputs = regexIn.exec(descripcion[indice]);
            if (isFormatoFecha) {
                // parse fecha si es que tiene dos logitudes
                var fecha = isFormatoFecha[0].split('-');
                if (fecha.length === 2) {
                    if (modificadores[indice] === '=') {
                        return {
                            categoria: 'equal',
                            valor: '"' +
                                etiqueta +
                                '"' +
                                ':{">=":"' +
                                new Date(fecha[0]).toISOString() +
                                '"}, "' +
                                etiqueta +
                                '":{"<=":"' +
                                new Date(fecha[1]).toISOString() +
                                '"}'
                        };
                    }
                    else if (modificadores[indice] === '+') {
                        return {
                            categoria: 'or-equal',
                            valor: '"' +
                                etiqueta +
                                '"' +
                                ':{">":"' +
                                new Date(fecha[0]).toISOString() +
                                '","<":"' +
                                new Date(fecha[1]).toISOString() +
                                '"}'
                        };
                    }
                    else {
                        return {
                            categoria: 'or',
                            valor: '{"' +
                                etiqueta +
                                '"' +
                                ':{">":"' +
                                new Date(fecha[0]).toISOString() +
                                '","<":"' +
                                new Date(fecha[1]).toISOString() +
                                '"}}'
                        };
                    }
                }
                else if (modificadores[indice].length > 0) {
                    if (modificadores[indice] === '=') {
                        return {
                            categoria: 'equal',
                            valor: '"' + etiqueta + '":"' + new Date(fecha[0]).toISOString() + '"'
                        };
                    }
                    else if (modificadores[indice] === '+') {
                        return {
                            categoria: 'or-equal',
                            valor: '"' + etiqueta + '":"' + new Date(fecha[0]).toISOString() + '"'
                        };
                    }
                    else {
                        return {
                            categoria: 'or',
                            valor: '{"' +
                                etiqueta +
                                '"' +
                                ':{"' +
                                modificadores[indice] +
                                '":"' +
                                new Date(fecha[0]).toISOString() +
                                '"}}'
                        };
                    }
                }
                else {
                    return {
                        categoria: 'or',
                        valor: '{"' + etiqueta + '"' + ':"' + new Date(fecha[0]).toISOString() + '"}'
                    };
                }
            }
            if (isMultipleInputs) {
                if (modificadores[indice] === '#') {
                    var inputs = descripcion[indice].split('&');
                    return {
                        categoria: 'in',
                        valor: '"' + etiqueta + '":{"in":[' + inputs.map(function (palabra) { return '"' + palabra + '"'; }) + ']}'
                    };
                }
            }
            // cuando se requiere que el campo sea igual, independiente de que sea
            // numero string
            if (modificadores[indice] === '=') {
                return {
                    categoria: 'equal',
                    valor: '"' + etiqueta + '":"' + descripcion[indice] + '"'
                };
            }
            // excluye valores NaN cuando no son numeros
            if (isFinite_1.default(parseFloat(descripcion[indice])) && modificadores[indice].length > 0) {
                if (modificadores[indice] === '+') {
                    return {
                        categoria: 'or-equal',
                        valor: '"' +
                            etiqueta +
                            '"' +
                            ':{"' +
                            modificadores[indice] +
                            '":' +
                            descripcion[indice] +
                            '}'
                    };
                }
                else {
                    return {
                        categoria: 'or',
                        valor: '{"' +
                            etiqueta +
                            '"' +
                            ':{"' +
                            modificadores[indice] +
                            '":' +
                            descripcion[indice] +
                            '}}'
                    };
                }
            }
            else if (isFinite_1.default(parseFloat(descripcion[indice]))) {
                if (modificadores[indice] === '+') {
                    return {
                        categoria: 'or-equal',
                        valor: '"' + etiqueta + '"' + ':' + descripcion[indice] + ''
                    };
                }
                else {
                    return {
                        categoria: 'or',
                        valor: '{"' + etiqueta + '"' + ':' + descripcion[indice] + '}'
                    };
                }
            }
            if (modificadores[indice] === '+') {
                return {
                    categoria: 'or-equal',
                    valor: '"' + etiqueta + '"' + ':{"contains":"' + descripcion[indice] + '"}'
                };
            }
            else {
                return {
                    categoria: 'or',
                    valor: '{"' + etiqueta + '"' + ':{"contains":"' + descripcion[indice] + '"}}'
                };
            }
        });
        var categoriaEqual = etiquetasConCategoria
            .filter(function (etiqueta) { return etiqueta.categoria === 'equal'; })
            .map(function (etiqueta) { return etiqueta.valor; });
        var categoriaOr = etiquetasConCategoria
            .filter(function (etiqueta) { return etiqueta.categoria === 'or'; })
            .map(function (etiqueta) { return etiqueta.valor; });
        var categoriaIn = etiquetasConCategoria
            .filter(function (etiqueta) { return etiqueta.categoria === 'in'; })
            .map(function (etiqueta) { return etiqueta.valor; });
        var categoriaOrEqual = etiquetasConCategoria
            .filter(function (etiqueta) { return etiqueta.categoria === 'or-equal'; })
            .map(function (etiqueta) { return etiqueta.valor; });
        var query = "where={\"or\":[]}";
        var auxQuery = '';
        if (categoriaEqual.length > 0) {
            auxQuery += categoriaEqual;
        }
        if (categoriaOr.length > 0) {
            if (auxQuery) {
                auxQuery += ',"or":[' + categoriaOr + ']';
            }
            else {
                auxQuery += '"or":[' + categoriaOr + ']';
            }
        }
        if (categoriaIn.length > 0) {
            if (auxQuery) {
                auxQuery += ',' + categoriaIn;
            }
            else {
                auxQuery += categoriaIn;
            }
        }
        if (categoriaOrEqual.length > 0) {
            if (auxQuery) {
                auxQuery += ',' + categoriaOrEqual;
            }
            else {
                auxQuery += categoriaOrEqual;
            }
        }
        if (auxQuery) {
            query = "where={" + auxQuery + "}";
        }
        else {
            query = '';
        }
        return query;
    };
    /**
     * Genera el query de busqueda a partir de un string
     *
     * @param {string} query
     * @returns {Query}
     * @memberof Query
     */
    Query.prototype.addQuery = function (query) {
        this._queryGenerador += " " + query;
        var etiquetas = this.parseQueryEtiquetas(query);
        var descripcion = this.parseQueryDescripcion(query);
        var modificadores = this.parseQueryModificador(query);
        if (etiquetas.length === descripcion.length && descripcion.length === modificadores.length) {
            this._etiquetas = this._etiquetas.concat(etiquetas);
            this._descripcion = this._descripcion.concat(descripcion);
            this._modificadores = this._modificadores.concat(modificadores);
            this._query = this.construccionQuery(this._etiquetas, this._descripcion, this._modificadores);
            return this;
        }
        else {
            this._query = '';
            return this;
        }
    };
    return Query;
}());
exports.default = Query;
//# sourceMappingURL=waterline-query-language-parse.js.map